package kinex;

import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.io.IOException;
import java.net.InetAddress;
import java.net.Socket;
import java.net.UnknownHostException;
import java.util.LinkedList;

public class KinectClient {
	
	public static KinectClient connect(String host, int port)
	throws UnknownHostException, IOException
	{
		InetAddress addr = InetAddress.getByName(host);
		return connect(addr, port);
	}
	
	public static KinectClient connect(InetAddress address, int port)
	throws IOException
	{
		final KinectClient client = new KinectClient();
		
		client.socket = new Socket(address, port);
		
		Thread thread = new Thread("KinectThread") {
			public void run() {
				client.recieve_func();
			}
		};
		
		client.thread = thread;
		client.thread.start();
		
		return client;
	}
	
	public void shutdown() {
		running = false;
		
		try {
			socket.close();
		} catch(IOException e) {
		}
		
		try {
			thread.join();
		} catch(InterruptedException e) {
		}
	}
	
	public KinectUser[] getUsers()
	{
		KinectUser[] users = null;
		synchronized(data_block) {
			users = build(data_block);
		}
		return users;
		
		//KinectUser[] temp;
		//synchronized(users) {
		//	temp = users;
		//}
		//return temp;
	}
	
	public KinectUser getUser(int index)
	{
		return getUsers()[index];
	}
	
	private boolean running = true;
	
	private Socket socket;
	
	private Thread thread;
	
	private LinkedList<String> data_block = new LinkedList<String>();
	
	//private KinectUser[] users;
	
	private void recieve_func() {
		BufferedReader br = null;
		try {
			br = new BufferedReader(
				new InputStreamReader(socket.getInputStream())
			);
		} catch(IOException e) {
			return;
		}
		
		try {
			String header = br.readLine();
		} catch(IOException e) {
		}
		
		LinkedList<String> strlist = new LinkedList<String>();
		while(running) {
			String line = null;
			try {
				line = br.readLine();
			} catch(IOException e) {
				break;
			}
			
			if(line == null) {
				System.out.println("Server connection shutdown");
				break;
			} else {
				if(line.startsWith("<Begin>")) {
					strlist = new LinkedList<String>();
					//strlist.clear();
				} else if(line.startsWith("<End>")) {
					synchronized(data_block) {
						data_block = strlist;
					}
					//KinectUser[] temp = build(strlist);
					//synchronized(users) {
					//	users = temp;
					//}
				} else {
					strlist.add(line);
				}
			}
		}
	}
	
	static KinectUser[] build(LinkedList<String> strlist) {
		KinectUser[] users = new KinectUser[2];
		users[0] = new KinectUser();
		users[1] = new KinectUser();
		
		int index = -1;
		for(String line : strlist) {
			String[] tokens = line.split(" ");
			
			if(tokens[0].equals("<User>")) {
				index++;
				
				int id = 0;
				try {
					id = Integer.parseInt(tokens[1]);
				} catch(NumberFormatException e) {
				}
				
				users[index].id = id;
			} else {
				if(tokens.length != 6) continue;
				
				float x, y, z;
				try {
					x = Float.parseFloat(tokens[1]);
					y = Float.parseFloat(tokens[2]);
					z = Float.parseFloat(tokens[3]);
				} catch(NumberFormatException e) {
					continue;
				}
				
				int sx, sy;
				try {
					sx = Integer.parseInt(tokens[4]);
					sy = Integer.parseInt(tokens[5]);
				} catch(NumberFormatException e) {
					continue;
				}
				
				writeJoint(tokens[0], x, y, z, sx, sy, users[index]);
			}
		}
		
		return users;
	}
	
	static void writeJoint(
		String tag, float x, float y, float z, int sx, int sy, KinectUser user
	)
	{
		if(tag.equals("<Head>")) {
			writeVector(x, y, z, user.Head);
			writeScreen(sx, sy, user.Head);
		} else if(tag.equals("<ShoulderCenter>")) {
			writeVector(x, y, z, user.ShoulderCenter);
			writeScreen(sx, sy, user.ShoulderCenter);
		} else if(tag.equals("<ShoulderLeft>")) {
			writeVector(x, y, z, user.ShoulderLeft);
			writeScreen(sx, sy, user.ShoulderLeft);
		} else if(tag.equals("<ShoulderRight>")) {
			writeVector(x, y, z, user.ShoulderRight);
			writeScreen(sx, sy, user.ShoulderRight);
		} else if(tag.equals("<ElbowLeft>")) {
			writeVector(x, y, z, user.ElbowLeft);
			writeScreen(sx, sy, user.ElbowLeft);
		} else if(tag.equals("<ElbowRight>")) {
			writeVector(x, y, z, user.ElbowRight);
			writeScreen(sx, sy, user.ElbowRight);
		} else if(tag.equals("<WristLeft>")) {
			writeVector(x, y, z, user.WristLeft);
			writeScreen(sx, sy, user.WristLeft);
		} else if(tag.equals("<WristRight>")) {
			writeVector(x, y, z, user.WristRight);
			writeScreen(sx, sy, user.WristRight);
		} else if(tag.equals("<HandLeft>")) {
			writeVector(x, y, z, user.HandLeft);
			writeScreen(sx, sy, user.HandLeft);
		} else if(tag.equals("<HandRight>")) {
			writeVector(x, y, z, user.HandRight);
			writeScreen(sx, sy, user.HandRight);
		} else if(tag.equals("<Spine>")) {
			writeVector(x, y, z, user.Spine);
			writeScreen(sx, sy, user.Spine);
		} else if(tag.equals("<HipCenter>")) {
			writeVector(x, y, z, user.HipCenter);
			writeScreen(sx, sy, user.HipCenter);
		} else if(tag.equals("<HipLeft>")) {
			writeVector(x, y, z, user.HipLeft);
			writeScreen(sx, sy, user.HipLeft);
		} else if(tag.equals("<HipRight>")) {
			writeVector(x, y, z, user.HipRight);
			writeScreen(sx, sy, user.HipRight);
		} else if(tag.equals("<KneeLeft>")) {
			writeVector(x, y, z, user.KneeLeft);
			writeScreen(sx, sy, user.KneeLeft);
		} else if(tag.equals("<KneeRight>")) {
			writeVector(x, y, z, user.KneeRight);
			writeScreen(sx, sy, user.KneeRight);
		} else if(tag.equals("<AnkleLeft>")) {
			writeVector(x, y, z, user.AnkleLeft);
			writeScreen(sx, sy, user.AnkleLeft);
		} else if(tag.equals("<AnkleRight>")) {
			writeVector(x, y, z, user.AnkleRight);
			writeScreen(sx, sy, user.AnkleRight);
		} else if(tag.equals("<FootLeft>")) {
			writeVector(x, y, z, user.FootLeft);
			writeScreen(sx, sy, user.FootLeft);
		} else if(tag.equals("<FootRight>")) {
			writeVector(x, y, z, user.FootRight);
			writeScreen(sx, sy, user.FootRight);
		}
	}
	
	static void writeVector(float x, float y, float z, KinectJoint v)
	{
		v.x = x;
		v.y = y;
		v.z = z;
	}
	
	static void writeScreen(int sx, int sy, KinectJoint v)
	{
		v.sx = sx;
		v.sy = sy;
	}
	
}
